<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 05/30/2004 *
 * ===================                          Last change: 08/09/2004 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : autopurge_functions.php                          *
 * -------------------------------------------------------------------- *
 * Short description : Special functions                                *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Spezielle Funktionen                             *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2016 by Mailer Developer Team                   *
 * For more information visit: http://mxchange.org                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	die();
} // END - if

// Add points in autopurge-mode
function addPointsAutoPurge ($userid, $points) {
	// Is the userid valid?
	if ((!isValidId($userid)) || ($points <= 0)) {
		// Please report all these bugs
		reportBug(__FUNCTION__, __LINE__, 'Invalid call. userid=' . $userid . ',points=' . $points);
	} // END - if

	// Add points over the ref system directly now
	initReferralSystem();
	addPointsThroughReferralSystem('autopurge_add', $userid, $points);

	// Send out mail to user
	$message = loadEmailTemplate('member_autopurge_points', array('points' => $points), $userid);
	sendEmail($userid, '{--MEMBER_AUTOPURGE_SUBJECT--}', $message);
}

// ----------------------------------------------------------------------------
//                     Wrapper for configuration entries
// ----------------------------------------------------------------------------

// Checks whether auto-purging is enabled
function isAutoPurgingActive () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = (getConfig('auto_purge_active') == 'Y');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Wrapper for 'autopurge_inactive
function getAutopurgeInactive () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = (getConfig('autopurge_inactive') == 'Y');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Checks whether purging of inactive accounts is enabled
function isAutopurgeInactiveEnabled () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = (getAutopurgeInactive() == 'Y');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for ap_inactive_time
function getApInactiveTime () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('ap_inactive_time');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for auto_purge
function getAutoPurge () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('auto_purge');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for ap_dm_timeout
function getApDmTimeout () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('ap_dm_timeout');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for ap_tasks_time
function getApTasksTime () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('ap_tasks_time');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for ap_unconfirmed_time
function getApUnconfirmedTime () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('ap_unconfirmed_time');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// [EOF]
?>
