<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 04/04/2009 *
 * ===================                          Last change: 04/04/2009 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : load_config.php                                  *
 * -------------------------------------------------------------------- *
 * Short description : Loader for configuration                         *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Lader fuer Konfiguration                         *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2016 by Mailer Developer Team                   *
 * For more information visit: http://mxchange.org                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	die();
} // END - if

// Set some config entries which is required for installation phase
setConfigEntry('OUTPUT_MODE' , 'render');
setConfigEntry('WRITE_FOOTER', 'Y');
setConfigEntry('_DB_TYPE'    , 'mysql');

// Mark configuration as NOT loaded which is the default
$GLOBALS['config_local_loaded'] = FALSE;

// Is the local configuration there?
if ((isIncludeReadable(getCachePath() . 'config-local.php')) && (isIncludeReadable('inc/config.php'))) {
	// We are better in installation mode
	$GLOBALS['__mailer_installing'] = TRUE;

	// Define default main_title here
	setConfigEntry('MAIN_TITLE', 'Your mail-exchange title');

	// Set output mode here
	setConfigEntry('OUTPUT_MODE', 'render');

	// Both exist! This is bad and should be avoided by the admin
	reportBug(__FILE__, __LINE__, 'You have uploaded or kept an out-dated file at <strong>inc/config.php</strong> along with the new file <strong>'.getCachePath().'config-local.php</strong>. Please remove <strong>inc/config.php</strong> to avoid incompatiblity issues. Thank you.');
} elseif (isIncludeReadable(getCachePath() . 'config-local.php')) {
	// Then load it
	loadIncludeOnce(getCachePath() . 'config-local.php');

	// Mark configuration as loaded
	$GLOBALS['config_local_loaded'] = TRUE;
} elseif (isIncludeReadable('inc/config.php')) {
	// Out-dated inc/config.php found, falling back to load and "wrap" it
	updateOldConfigFile();
} elseif (isInstalling()) {
	// Set some essential constants
	// @TODO Rewrite them to avoid this else block
	setConfigEntry('MAIN_TITLE', 'Your mail-exchanger title');
	setConfigEntry('SLOGAN'    , 'Your cool slogan here');
	setConfigEntry('WEBMASTER' , 'you@some-hoster.example');

	// Set output mode here
	setConfigEntry('OUTPUT_MODE', 'render');
}

// Check if the user setups his database login stuff...
if ((empty($GLOBALS['mysql']['login'])) && (!isInstalling()) && (!isInstaller()) && (!isGetRequestElementSet('installing')) && (isInstalled())) {
	// No login entered and outside installation mode
	outputHtml('<strong>{--MAILER_WARNING--}</strong>');
	if (isInstalled()) {
		// You have changed my configuration file!
		reportBug(__FILE__, __LINE__, '{--DIE_CONFIG_CHANGED_YOU--}');
	} else {
		// Please run the installation script (maybe again)
		reportBug(__FILE__, __LINE__, '{--DIE_RUN_INSTALLER_MYSQL--}');
	}
} elseif ((!isInstalling()) && (!isInstaller()) && (empty($GLOBALS['mysql']['password'])) && (getConfig('WARN_NO_PASS') == 'Y')) {
	// No database password entered
	displayMessage('<div class="warning">{--MAILER_WARNING--}</div> {--WARN_NULL_PASSWORD--}');
}

// [EOF]
?>
