<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 10/29/2003 *
 * ===================                          Last change: 01/06/2006 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : order.php                                        *
 * -------------------------------------------------------------------- *
 * Short description : Submits your order to the administrators         *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Sendet die Buchung an die Administratoren        *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2016 by Mailer Developer Team                   *
 * For more information visit: http://mxchange.org                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
$url = '';
if (!defined('__SECURITY')) {
	exit();
} elseif ((!isExtensionActive('order')) && (!isAdmin())) {
	displayMessage('{%pipe,generateExtensionInactiveNotInstalledMessage=order%}');
	return;
} elseif (!isMember()) {
	// Sorry, no guest access!
	$url = 'modules.php?module=index';
} elseif (!isGetRequestElementSet('order')) {
	// You cannot call this module directly!
	$url = 'modules.php?module=login&amp;what=order';
}

// When URL is empty nothing bad happend here
if (empty($url)) {
	// Auto-send is inactive
	$content['admin_autosend']  = '{--ADMIN_AUTOSEND_INACTIVE--}';
	$content['member_autosend'] = '{--MEMBER_AUTOSEND_INACTIVE--}';
	$type = 'ADMIN';

	// Is the auto-send mechanism active or inactive?
	if (isAutosendMemberMailsEnabled()) {
		// Auto-send is active
		$content['admin_autosend']  = '{--ADMIN_AUTOSEND_ACTIVE--}';
		$content['member_autosend'] = '{--MEMBER_AUTOSEND_ACTIVE--}';
		$type = 'NEW';
	} // END - if

	// Update pool data and check if the entry is valid
	if (!updatePoolDataById(getRequestElement('order'), 'data_type', $type, NULL, sprintf(" AND `sender`=%s AND `data_type`='TEMP'", getMemberId()))) {
		// Merge array with pool data
		// @TODO Unused: timestamp
		$content = merge_array($content, getPoolDataFromId(getRequestElement('order')));

		// Fix empty subject line
		if (empty($content['subject'])) {
			$content['subject'] = '{--DEFAULT_SUBJECT--}';
		} // END - if

		// Calculate used points
		$content['payed_points'] = $content['target_send'] * getPaymentPrice($content['payment_id']);

		// Subtract them from the user's account and ignore return status
		subtractPoints('order', getMemberId(), $content['payed_points']);

		// Update used points
		$add = '';
		if ((isExtensionInstalledAndNewer('order', '0.1.1')) && (getOrderMaxFull() == 'ORDER')) {
			$add = ',`mail_orders`=`mail_orders`+1';
		} // END - if

		// Send an email to the user
		$message_mem = loadEmailTemplate('member_order_normal', $content, getMemberId());
		sendEmail(getMemberId(), '{--MEMBER_NEW_QUEUE--}', $message_mem);

		// Notify admins about this
		sendAdminNotification('{--ADMIN_NEW_QUEUE--}', 'admin_order_normal', $content, getMemberId());

		// Create new task (we ignore the task id here)
		createNewTask(
			'{--ADMIN_NEW_QUEUE--}',
			'<pre>' . loadEmailTemplate('admin_order_normal', $content, getMemberId()) . '</pre>',
			'MEMBER_ORDER',
			getMemberId(),
			0,
			false
		);

		// Output back bottom
		loadTemplate('member_order_thanks');
	} else {
		// Matching line not found or already 'placed' in send queue
		redirectToUrl('modules.php?module=login');
	}
} else {
	// Redirect...
	redirectToUrl($url);
}

// [EOF]
?>
